# -*- coding: utf-8 -*-
#
#       main.py
#
#       Copyright © 2011, 2014 Brandon Invergo <brandon@invergo.net>
#
#       This file is part of Grotesque.
#
#       Grotesque is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Grotesque is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with Grotesque.  If not, see <http://www.gnu.org/licenses/>.


#from collections import Counter

from gi.repository import Gtk, GdkPixbuf


class FilterStore(Gtk.ListStore):
    '''This class implements a widget which holds information on a library
    filter. It keeps count of the number of occurrances of each item in the
    library. When, for example, there are no more stories by Author X in the
    library, Author X should be removed from the filter.

    '''
    def __init__(self, pixbuf=False):
        if pixbuf:
            super(FilterStore, self).__init__(GdkPixbuf.Pixbuf, str)
        else:
            super(FilterStore, self).__init__(str)
        self.counter = dict()  # Counter()
        self.pixbuf = pixbuf

    def add_item(self, item_str, pixbuf=None):
        self.counter_incr(item_str)
        if self.counter[item_str] == 1:
            if pixbuf:
                self.append([pixbuf, item_str])
            else:
                self.append([item_str])

    def remove_item(self, item_str):
        if not item_str:
            return False
        if self.pixbuf:
            id_column = 1
        else:
            id_column = 0
        self.counter_decr(item_str)
        if self.counter[item_str] == 0:
            row_iter = self.get_iter_first()
            while row_iter:
                if self.get_value(row_iter, id_column) == item_str:
                    self.remove(row_iter)
                    return True
                row_iter = self.iter_next(row_iter)
        return False

    def counter_decr(self, item_str):
        if (item_str not in self.counter or
                self.counter[item_str] == 0):
            return False
        self.counter[item_str] -= 1

    def counter_incr(self, item_str):
        if item_str in self.counter:
            self.counter[item_str] += 1
        else:
            self.counter[item_str] = 1

    def serialize(self):
        '''This method creates serializable versions of the data for saving.

        '''
        rows = []
        row_iter = self.get_iter_first()
        while row_iter:
            if self.pixbuf:
                row = list(self.get(row_iter, 0, 1))
                row[0] = None
                rows.append(row)
            else:
                row = self.get(row_iter, 0)
                if row != ('(All)',):
                    rows.append(self.get(row_iter, 0))
            row_iter = self.iter_next(row_iter)
        return (rows, self.counter)

    def deserialize(self, data, pixbufs=None):
        '''This method takes serialized data and inserts it in the store.

        '''
        rows, counter = data
        self.counter = counter
        for row in rows:
            if pixbufs is None:
                if row == ('(All)',):
                    continue
                self.append(row)
            else:
                row = list(row)
                row[0] = pixbufs[row[1]]
                self.append(row)
