# -*- coding: utf-8 -*-
#
#       library.py
#
#       Copyright © 2011, 2014 Brandon Invergo <brandon@invergo.net>
#
#       This file is part of Grotesque.
#
#       Grotesque is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Grotesque is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with Grotesque.  If not, see <http://www.gnu.org/licenses/>.

import threading

from gi.repository import Gtk, Gdk, GObject

from dialogs.progressdialog import ProgressDialog
from treatyofbabel import ifdb, storydata


def update_library(parent, library, xml_library, coverart_dir):
        d = Gtk.MessageDialog(parent, Gtk.DialogFlags.MODAL,
                              Gtk.MessageType.INFO,
                              Gtk.ButtonsType.OK)
        d_msg1 = \
            'An old library format has been detected and will now be updated.'
        d.set_markup(d_msg1)
        d.set_title('Grotesque')
        edit_response = d.run()
        d.destroy()
        with open(xml_library, 'r') as xml_handle:
            library.load_ifiction(xml_handle)
        d = Gtk.MessageDialog(parent, Gtk.DialogFlags.MODAL,
                              Gtk.MessageType.QUESTION,
                              Gtk.ButtonsType.YES_NO)
        d.set_markup('This version of Grotesque supports more metadata than '
                     'older versions and may also display cover art and '
                     'ratings from IFDB.\n\nWould you like this data to be '
                     'fetched for the stories in your library now? (You may '
                     'also do this later via the IFDB query feature of the '
                     'Edit dialog.)')
        d.set_title('Grotesque')
        edit_response = d.run()
        d.destroy()
        if edit_response == Gtk.ResponseType.YES:
            update_dialog = ProgressDialog("Updating...", parent)
            update_thread = StoryUpdateThread(library, coverart_dir,
                                              update_dialog)
            updating = update_thread.update_stories()
            GObject.idle_add(updating.next)
            update_response = update_dialog.run()
            update_dialog.destroy()
            if (update_response == Gtk.ResponseType.REJECT or
                    update_response == Gtk.ResponseType.DELETE_EVENT):
                update_thread.stop()
        return True


class StoryUpdateThread(threading.Thread):
    def __init__(self, library, coverart_dir, dialog):
        threading.Thread.__init__(self)
        self.library = library
        self.dialog = dialog
        self.coverart_dir = coverart_dir
        self.stopthread = False

    def stop(self):
        self.stopthread = True

    def update_stories(self):
        row_iter = self.library.list_store.get_iter_first()
        while row_iter:
            if self.stopthread:
                yield False
            # Get the story
            story = self.library.get_story(row_iter)
            temp_story = storydata.StoryData()
            temp_story.title = story.title
            temp_story.author = story.author
            temp_story.genre = story.genre
            temp_story.ifid = story.ifid
            temp_story.story_file = story.story_file
            temp_story.format = story.format
            temp_story.description = story.description
            Gdk.threads_leave()
            xml = ifdb.fill_meta_from_ifdb(temp_story)
            yield True
            ifdb.fetch_cover_art(temp_story, self.coverart_dir, xml)
            biblio = {}
            biblio['Title'] = temp_story.title
            biblio['Author'] = temp_story.author
            biblio['Language'] = temp_story.language
            biblio['Headline'] = temp_story.headline
            biblio['First published'] = temp_story.firstpublished
            biblio['Genre'] = temp_story.genre
            biblio['Group'] = temp_story.group
            biblio['Series'] = temp_story.series
            biblio['Series number'] = temp_story.series
            biblio['Forgiveness'] = temp_story.forgiveness
            biblio['Description'] = temp_story.description
            biblio['Rating'] = temp_story.starrating
            biblio['Format'] = temp_story.format
            biblio['IFID'] = temp_story.ifid
            biblio['TUID'] = temp_story.tuid
            self.library.edit_entry(row_iter, biblio)
            # Advance the progress bar.
            self.dialog.progressbar.pulse()
            row_iter = self.library.list_store.iter_next(row_iter)
            yield True
        self.dialog.response(Gtk.ResponseType.OK)
        yield False
